@@ -44,9 +44,7 @@
 
 #include "keyboard-layout.h"
 
-static void xfce_keyboard_layout_helper_class_init                (XfceKeyboardLayoutHelperClass *klass);
-static void xfce_keyboard_layout_helper_init                      (XfceKeyboardLayoutHelper      *helper);
-static void xfce_keyboard_layout_helper_finalize                  (GObject                       *object);
+static void xfce_keyboard_layout_helper_process_xmodmap           (void);
 static void xfce_keyboard_layout_helper_set_model                 (XfceKeyboardLayoutHelper      *helper);
 static void xfce_keyboard_layout_helper_set_layout                (XfceKeyboardLayoutHelper      *helper);
 static void xfce_keyboard_layout_helper_set_variant               (XfceKeyboardLayoutHelper      *helper);
@@ -66,7 +64,8 @@ struct _XfceKeyboardLayoutHelper
 
     /* xfconf channel */
     XfconfChannel     *channel;
-    gboolean xkb_disable_settings;
+
+    gboolean           xkb_disable_settings;
 
 #ifdef HAVE_LIBXKLAVIER
     /* libxklavier */
@@ -81,10 +80,7 @@ G_DEFINE_TYPE (XfceKeyboardLayoutHelper, xfce_keyboard_layout_helper, G_TYPE_OBJ
 static void
 xfce_keyboard_layout_helper_class_init (XfceKeyboardLayoutHelperClass *klass)
 {
-    GObjectClass *gobject_class;
 
-    gobject_class = G_OBJECT_CLASS (klass);
-    gobject_class->finalize = xfce_keyboard_layout_helper_finalize;
 }
 
 static void
@@ -94,7 +90,7 @@ xfce_keyboard_layout_helper_init (XfceKeyboardLayoutHelper *helper)
     helper->channel = NULL;
 
     /* open the channel */
-    helper->channel = xfconf_channel_new ("keyboard-layout");
+    helper->channel = xfconf_channel_get ("keyboard-layout");
 
     /* monitor channel changes */
     g_signal_connect (G_OBJECT (helper->channel), "property-changed", G_CALLBACK (xfce_keyboard_layout_helper_channel_property_changed), helper);
@@ -110,18 +106,34 @@ xfce_keyboard_layout_helper_init (XfceKeyboardLayoutHelper *helper)
     xfce_keyboard_layout_helper_set_model (helper);
     xfce_keyboard_layout_helper_set_layout (helper);
     xfce_keyboard_layout_helper_set_variant (helper);
+
+    xfce_keyboard_layout_helper_process_xmodmap ();
 }
 
+
+
 static void
-xfce_keyboard_layout_helper_finalize (GObject *object)
+xfce_keyboard_layout_helper_process_xmodmap (void)
 {
-    XfceKeyboardLayoutHelper *helper = XFCE_KEYBOARD_LAYOUT_HELPER (object);
+    const gchar *xmodmap_path;
 
-    /* release the channel */
-    if (G_LIKELY (helper->channel))
-        g_object_unref (G_OBJECT (helper->channel));
+    xmodmap_path = g_build_filename (xfce_get_homedir (), ".Xmodmap", NULL);
 
-    (*G_OBJECT_CLASS (xfce_keyboard_layout_helper_parent_class)->finalize) (object);
+    if (g_file_test (xmodmap_path, G_FILE_TEST_EXISTS))
+    {
+        /* There is a .Xmodmap file, try to use it */
+        const gchar *xmodmap_command;
+        GError      *error = NULL;
+
+        xmodmap_command = g_strconcat ("xmodmap ", xmodmap_path, NULL);
+
+        /* Launch the xmodmap command and only print errors when in debugging mode */
+        if (!g_spawn_command_line_async (xmodmap_command, &error))
+        {
+            DBG ("Xmodmap call failed: %s", error->message);
+            g_error_free (error);
+        }
+    }
 }
 
 static void
@@ -210,4 +222,6 @@ xfce_keyboard_layout_helper_channel_property_changed (XfconfChannel      *channe
     {
         xfce_keyboard_layout_helper_set_variant (helper);
     }
+
+    xfce_keyboard_layout_helper_process_xmodmap ();
 }
