@@ -37,7 +37,8 @@
 
 #include <dbus/dbus-glib.h>
 
-#include <libxfcegui4/libxfcegui4.h>
+#include <libxfce4ui/libxfce4ui.h>
+#include <libxfce4util/libxfce4util.h>
 #include <xfconf/xfconf.h>
 #include <libxfce4kbd-private/xfce-shortcuts-provider.h>
 #include <libxfce4kbd-private/xfce-shortcuts-grabber.h>
@@ -54,8 +55,6 @@ enum
 
 
 
-static void            xfce_keyboard_shortcuts_helper_class_init         (XfceKeyboardShortcutsHelperClass *klass);
-static void            xfce_keyboard_shortcuts_helper_init               (XfceKeyboardShortcutsHelper      *helper);
 static void            xfce_keyboard_shortcuts_helper_finalize           (GObject                          *object);
 static void            xfce_keyboard_shortcuts_helper_get_property       (GObject                          *object,
                                                                           guint                             prop_id,
@@ -73,6 +72,7 @@ static void            xfce_keyboard_shortcuts_helper_shortcut_removed   (XfceSh
                                                                           XfceKeyboardShortcutsHelper      *helper);
 static void            xfce_keyboard_shortcuts_helper_shortcut_activated (XfceShortcutsGrabber             *grabber,
                                                                           const gchar                      *shortcut,
+                                                                          gint                              timestamp,
                                                                           XfceKeyboardShortcutsHelper      *helper);
 static void            xfce_keyboard_shortcuts_helper_load_shortcuts     (XfceKeyboardShortcutsHelper      *helper);
 
@@ -96,36 +96,7 @@ struct _XfceKeyboardShortcutsHelper
 
 
 
-static GObjectClass *xfce_keyboard_shortcuts_helper_parent_class = NULL;
-
-
-
-GType
-xfce_keyboard_shortcuts_helper_get_type (void)
-{
-  static GType type = G_TYPE_INVALID;
-
-  if (G_UNLIKELY (type == G_TYPE_INVALID))
-    {
-      static const GTypeInfo info =
-      {
-        sizeof (XfceKeyboardShortcutsHelperClass),
-        NULL,
-        NULL,
-        (GClassInitFunc) xfce_keyboard_shortcuts_helper_class_init,
-        NULL,
-        NULL,
-        sizeof (XfceKeyboardShortcutsHelper),
-        0,
-        (GInstanceInitFunc) xfce_keyboard_shortcuts_helper_init,
-        NULL,
-      };
-
-      type = g_type_register_static (G_TYPE_OBJECT, "XfceKeyboardShortcutsHelper", &info, 0);
-    }
-
-  return type;
-}
+G_DEFINE_TYPE (XfceKeyboardShortcutsHelper, xfce_keyboard_shortcuts_helper, G_TYPE_OBJECT)
 
 
 
@@ -134,9 +105,6 @@ xfce_keyboard_shortcuts_helper_class_init (XfceKeyboardShortcutsHelperClass *kla
 {
   GObjectClass *gobject_class;
 
-  /* Determine the parent type class */
-  xfce_keyboard_shortcuts_helper_parent_class = g_type_class_peek_parent (klass);
-
   gobject_class = G_OBJECT_CLASS (klass);
   gobject_class->finalize = xfce_keyboard_shortcuts_helper_finalize;
   gobject_class->get_property = xfce_keyboard_shortcuts_helper_get_property;
@@ -218,7 +186,7 @@ xfce_keyboard_shortcuts_helper_set_property (GObject      *object,
 
 
 
-static void 
+static void
 xfce_keyboard_shortcuts_helper_shortcut_added (XfceShortcutsProvider       *provider,
                                                const gchar                 *shortcut,
                                                XfceKeyboardShortcutsHelper *helper)
@@ -273,13 +241,13 @@ xfce_keyboard_shortcuts_helper_load_shortcuts (XfceKeyboardShortcutsHelper *help
 static void
 xfce_keyboard_shortcuts_helper_shortcut_activated (XfceShortcutsGrabber        *grabber,
                                                    const gchar                 *shortcut,
+                                                   gint                         timestamp,
                                                    XfceKeyboardShortcutsHelper *helper)
 {
-  XfceShortcut *sc;
-  GdkDisplay   *display;
-  GdkScreen    *screen;
-  GError       *error = NULL;
-  gint          monitor;
+  XfceShortcut  *sc;
+  GError        *error = NULL;
+  gchar        **argv;
+  gboolean       succeed;
 
   g_return_if_fail (XFCE_IS_KEYBOARD_SHORTCUTS_HELPER (helper));
   g_return_if_fail (XFCE_IS_SHORTCUTS_PROVIDER (helper->provider));
@@ -296,20 +264,27 @@ xfce_keyboard_shortcuts_helper_shortcut_activated (XfceShortcutsGrabber        *
   if (G_UNLIKELY (sc == NULL))
     return;
 
-  display = gdk_display_get_default ();
+  DBG ("command = %s, snotify = %s (time = %d)",
+       sc->command, sc->snotify ? "true" : "false", timestamp);
 
-  DBG ("command = %s", sc->command);
+  /* Handle the argv ourselfs, because xfce_spawn_command_line_on_screen() does
+   * not accept a custom timestamp for startup notification */
+  succeed = g_shell_parse_argv (sc->command, NULL, &argv, &error);
+  if (G_LIKELY (succeed))
+    {
+      succeed = xfce_spawn_on_screen_with_child_watch (xfce_gdk_screen_get_active (NULL),
+                                                       NULL, argv, NULL,
+                                                       G_SPAWN_SEARCH_PATH, sc->snotify,
+                                                       timestamp, NULL, NULL, &error);
 
-  /* Determine active monitor */
-  screen = xfce_gdk_display_locate_monitor_with_pointer (display, &monitor);
+      g_strfreev (argv);
+    }
 
-  /* Spawn command */
-  if (!G_UNLIKELY (!xfce_gdk_spawn_command_line_on_screen (screen, sc->command, &error)))
-    if (G_LIKELY (error != NULL))
-      {
-        g_error ("%s", error->message);
-        g_error_free (error);
-      }
+  if (!succeed)
+    {
+      g_warning ("Failed to spawn command \"%s\": %s", sc->command, error->message);
+      g_error_free (error);
+    }
 
   xfce_shortcut_free (sc);
 }
